package at.ac.uibk.gitsearch.repository.search;


import at.ac.uibk.gitsearch.service.dto.MetaDataDto;
import at.ac.uibk.gitsearch.service.dto.SearchResultDto;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Repository;

import java.io.IOException;


@Repository
public class TotalRepositoryImpl implements TotalRepository {

    private final RestHighLevelClient elasticsearchClient;

    public TotalRepositoryImpl(RestHighLevelClient elasticsearchClient) {
        this.elasticsearchClient = elasticsearchClient;
    }

    @Override
    public SearchResultDto customSearchWithHighlightedSummary(String query) throws IOException {
        SearchRequest searchRequest = new SearchRequest(HighlightRepositoryConstants.INDEX);

        BoolQueryBuilder matchQueryBuilder1 = QueryBuilders.boolQuery()
            .must(QueryBuilders.queryStringQuery(query).defaultField(HighlightRepositoryConstants.CONTENT_FIELD));

        TermsAggregationBuilder groupSubGroup = AggregationBuilders
            .terms(HighlightRepositoryConstants.SUB_GROUP_AGG)
            .field(HighlightRepositoryConstants.SUB_GROUP_FIELD);

        TermsAggregationBuilder groupFileExtension = AggregationBuilders
            .terms(HighlightRepositoryConstants.FILE_EXTENSION_AGG)
            .field(HighlightRepositoryConstants.FILE_EXTENSION_FIELD);

        TermsAggregationBuilder groupRepository = AggregationBuilders
            .terms(HighlightRepositoryConstants.REPOSITORY_AGG)
            .field(HighlightRepositoryConstants.REPOSITORY_FIELD);

        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.query(matchQueryBuilder1)
            .size(0)
            .aggregation(groupSubGroup)
            .aggregation(groupFileExtension)
            .aggregation(groupRepository);

        searchRequest.source(sourceBuilder);

        SearchResponse searchResponse = elasticsearchClient.search(searchRequest, RequestOptions.DEFAULT);

        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

        MetaDataDto metaDataDto = new MetaDataDto(searchResponse.getAggregations());

        long hitCount = searchResponse.getHits().getTotalHits();

        return new SearchResultDto(null, metaDataDto, hitCount);
    }
}
