package at.ac.uibk.gitsearch.service.dto;

import at.ac.uibk.gitsearch.repository.search.HighlightRepositoryConstants;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class MetaDataDto {
    private List<FrequencyDto<String>> fileFormat;
    private List<FrequencyDto<String>> repositories;
    private List<FrequencyDto<String>> university;

    public MetaDataDto(Aggregations aggregations) {
        Terms terms = aggregations.get(HighlightRepositoryConstants.SUB_GROUP_AGG);
        Collection<Terms.Bucket> buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.university = buckets.stream().map(bucket -> new FrequencyDto<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());

        terms = aggregations.get(HighlightRepositoryConstants.REPOSITORY_AGG);
        buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.repositories = buckets.stream().map(bucket -> new FrequencyDto<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());

        terms = aggregations.get(HighlightRepositoryConstants.FILE_EXTENSION_AGG);
        buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.fileFormat = buckets.stream().map(bucket -> new FrequencyDto<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());
    }

    public List<FrequencyDto<String>> getFileFormat() {
        return fileFormat;
    }

    public void setFileFormat(List<FrequencyDto<String>> fileFormat) {
        this.fileFormat = fileFormat;
    }

    public List<FrequencyDto<String>> getRepositories() {
        return repositories;
    }

    public void setRepositories(List<FrequencyDto<String>> repositories) {
        this.repositories = repositories;
    }

    public List<FrequencyDto<String>> getUniversity() {
        return university;
    }

    public void setUniversity(List<FrequencyDto<String>> university) {
        this.university = university;
    }
}
