package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.repository.search.HighlightRepository;
import at.ac.uibk.gitsearch.repository.search.SearchResultSearchRepository;
import at.ac.uibk.gitsearch.repository.search.TotalRepository;
import at.ac.uibk.gitsearch.service.dto.SearchResultDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.List;
//import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * REST controller for managing {@link DocumentInfo}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class SearchResultResource {

    private final Logger log = LoggerFactory.getLogger(SearchResultResource.class);

    private static final String ENTITY_NAME = "searchResult";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SearchResultSearchRepository searchResultSearchRepository;
    private final HighlightRepository highlightRepository;
    private final TotalRepository totalRepository;

    public SearchResultResource(SearchResultSearchRepository searchResultSearchRepository, HighlightRepository highlightRepository, TotalRepository totalRepository) {
        this.searchResultSearchRepository = searchResultSearchRepository;
        this.highlightRepository = highlightRepository;
        this.totalRepository = totalRepository;
    }

    /**
     * {@code GET  /search-results} : get all the searchResults.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of searchResults in body.
     */
    @GetMapping("/search-results")
    public List<DocumentInfo> getAllSearchResults() {
        log.debug("REST request to get all SearchResults");
        return searchResultSearchRepository.findByContent("Assignment 1");
    }

    /**
     * //     * {@code SEARCH  /_search/search-results?query=:query} : search for the searchResult corresponding
     * //     * to the query.
     * //     *
     * //     * @param query the query of the searchResult search.
     * //     * @return the result of the search.
     * //
     */
    @GetMapping("/_search/search-results")
    public SearchResultDto searchSearchResults(@RequestParam String query, @RequestParam int pageSize, @RequestParam int page, @RequestParam List<String> selectedRepository, @RequestParam List<String> selectedUniversity, @RequestParam List<String> selectedFileFormat) throws IOException {
        log.debug("REST request to search SearchResults for query {}, page {}, pageSize {}, selectedRepository {}, selectedUniversity {}, selectedFileFormat {}", query, page, pageSize, selectedRepository, selectedUniversity, selectedFileFormat);
        // return searchResultSearchRepository.findByContent(query);
        return highlightRepository.customSearchWithHighlightedSummary(query, pageSize, page, selectedRepository, selectedUniversity, selectedFileFormat);
    }

    /**
     * //     * {@code SEARCH  /_search/search-results?query=:query} : search for the searchResult corresponding
     * //     * to the query.
     * //     *
     * //     * @param query the query of the searchResult search.
     * //     * @return the result of the search.
     * //
     */
    @GetMapping("/_search/search-results-total")
    public SearchResultDto searchSearchResultsTotal(@RequestParam String query) throws IOException {
        log.debug("REST request to search searchSearchResultsTotal for query {}", query);
        // return searchResultSearchRepository.findByContent(query);
        return totalRepository.customSearchWithHighlightedSummary(query);
    }
}
