import {Injectable} from '@angular/core';
import {HttpClient, HttpResponse} from '@angular/common/http';
import {Observable} from 'rxjs';

import {SERVER_API_URL} from 'app/app.constants';
import {createRequestOption} from 'app/shared/util/request-util';
import {ISearchResult} from 'app/shared/model/search-result.model';

type EntityResponseType = HttpResponse<ISearchResult>;

@Injectable({providedIn: 'root'})
export class SearchResultService {
  public resourceUrl = SERVER_API_URL + 'api/search-results';
  public resourceSearchUrl = SERVER_API_URL + 'api/_search/search-results';
  public resourceSearchTotalUrl = SERVER_API_URL + 'api/_search/search-results-total';

  constructor(protected http: HttpClient) {
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http
      .get<ISearchResult>(`${this.resourceUrl}/${id}`, {observe: 'response'})
      .pipe();
  }

  query(req?: any): Observable<EntityResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<ISearchResult>(this.resourceUrl, {params: options, observe: 'response'})
      .pipe();
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, {observe: 'response'});
  }

  search(req: { query: string; pageSize: number;  page: number; selectedRepository: string[]; selectedUniversity: string[]; selectedFileFormat: string[];  }): Observable<EntityResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<ISearchResult>(this.resourceSearchUrl, {params: options, observe: 'response'})
      .pipe();
  }

  searchTotals(req: { query: string }): Observable<EntityResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<ISearchResult>(this.resourceSearchTotalUrl, {params: options, observe: 'response'})
      .pipe();
  }

  /*
  protected convertDateFromClient(searchResult: ISearchResult): ISearchResult {
    const copy: ISearchResult = Object.assign({}, searchResult, {
      lastModified: searchResult.lastModified && searchResult.lastModified.isValid() ? searchResult.lastModified.toJSON() : undefined,
      lastAccessed: searchResult.lastAccessed && searchResult.lastAccessed.isValid() ? searchResult.lastAccessed.toJSON() : undefined,
      indexingDate: searchResult.indexingDate && searchResult.indexingDate.isValid() ? searchResult.indexingDate.toJSON() : undefined,
    });
    return copy;
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.lastModified = res.body.lastModified ? moment(res.body.lastModified) : undefined;
      res.body.lastAccessed = res.body.lastAccessed ? moment(res.body.lastAccessed) : undefined;
      res.body.indexingDate = res.body.indexingDate ? moment(res.body.indexingDate) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((searchResult: ISearchResult) => {
        searchResult.lastModified = searchResult.lastModified ? moment(searchResult.lastModified) : undefined;
        searchResult.lastAccessed = searchResult.lastAccessed ? moment(searchResult.lastAccessed) : undefined;
        searchResult.indexingDate = searchResult.indexingDate ? moment(searchResult.indexingDate) : undefined;
      });
    }
    return res;
  }
  */
}
